package com.localtransfer;

import android.app.PendingIntent;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.core.app.NotificationCompat;
import androidx.core.app.NotificationManagerCompat;

import java.util.ArrayList;
import java.util.ConcurrentModificationException;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class Progress {

    public static final int UPLOAD = 1000;
    public static final int DOWNLOAD = 1001;


    private static NotificationCompat.Builder bl = new NotificationCompat.Builder(Transfer.activity, null);
    private static NotificationManagerCompat notifiManager = NotificationManagerCompat.from(Transfer.activity);
    private static final LayoutInflater inflater = (LayoutInflater) Transfer.activity.getSystemService(Transfer.activity.LAYOUT_INFLATER_SERVICE);

    private static List instances = new ArrayList();

    private Integer id;
    private String name;
    private long size;

    public long loaded;
    public long percent;

    public static View root;
    public static boolean fragment_on = false;
    public static boolean app_started = false;

    private Timer timer = new Timer();
    private int drawable;
    private String info;

    public Button button;
    private boolean run = false;

    public Progress(String name, long size, int type) {

        instances.add(this);

        run = true;

        this.name = name;
        this.size = size;

        id = View.generateViewId();

        switch (type) {
            case Progress.DOWNLOAD:
                drawable = R.drawable.ic_download_24;
                info = "Download complete";
                break;
            case Progress.UPLOAD:
                drawable = R.drawable.ic_upload_24;
                info = "Upload complete";
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + type);
        }

        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                showProgress();
            }
        }, 0, 1000);
    }

    public static List getInstances() {
        return instances;
    }

    public void stopProgress() {
        timer.cancel();
        showProgress();

        run = false;

        if(! app_started) {
            bl.setSmallIcon(R.drawable.ic_upload_and_download_from_the_cloud)
                    .setContentTitle(name)
                    .setContentText(info)
                    .setProgress(0, 0, false)
                    .setAutoCancel(true)
                    .setOngoing(false);
            notifiManager.notify(id, bl.build());
        }
    }

    public void delete() {
        instances.remove(this);
    }

    public static Progress getProgress(int id) {

        for (Object obj: instances) {
            Progress p = (Progress) obj;
            if (p.id == id) return p;
        }
        return null;
    }

    public static boolean setButton(String name, Button button) {

        for (Object obj: instances) {
            Progress p = (Progress) obj;
            if (name.equals(p.name) && p.run) {
                p.button = button;
                return true;
            }
        }
        return false;
    }


    public void showProgress(){

        try {

            String HumanLoaded = Transfer.humanReadableByteCountBin(loaded);
            String HumanSize = Transfer.humanReadableByteCountBin(size);

            if(app_started) {
                notifiManager.cancel(id);
            }
            else {

                Intent intent = new Intent(Transfer.activity, MainActivity.class);
                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                PendingIntent pendingIntent = PendingIntent.getActivity(Transfer.activity, 0, intent, 0);

                bl.setSmallIcon(R.drawable.ic_upload_and_download_from_the_cloud)
                        .setContentTitle(name)
                        .setContentText(String.format("%d%% %s/%s", percent, HumanLoaded, HumanSize))
                        .setProgress(100, (int) percent, false)
                        .setContentIntent(pendingIntent)
                        .setAutoCancel(true)
                        .setOngoing(true);
                notifiManager.notify(id, bl.build());
            }

            if(app_started && fragment_on) {
                LinearLayout groot = root.findViewById(R.id.groot);

                final View progress;

                if (groot.findViewById(id) != null) {
                    progress = groot.findViewById(id);
                } else {
                    progress = inflater.inflate(R.layout.progress, null);
                    progress.setId(id);
                    Transfer.activity.runOnUiThread(() -> {
                        groot.addView(progress, 0);
                    });
                }

                Transfer.activity.runOnUiThread(() -> {
                    ((ProgressBar) progress.findViewById(R.id.progressBar)).setProgress((int) percent);
                    ((ImageView) progress.findViewById(R.id.transferType)).setImageResource(drawable);
                    ((TextView) progress.findViewById(R.id.progressText)).setText(percent + " %");
                    ((TextView) progress.findViewById(R.id.fileName)).setText(name);
                    ((TextView) progress.findViewById(R.id.fileSize)).setText(String.format("%s/%s", HumanLoaded, HumanSize));
                });

            }

        } catch (ConcurrentModificationException e) {
            e.printStackTrace();
        }

    }

}
