package com.localtransfer.fragment;

import android.animation.ObjectAnimator;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;

import androidx.core.content.FileProvider;
import androidx.fragment.app.Fragment;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import android.os.Environment;
import android.view.ContextMenu;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.google.android.material.snackbar.Snackbar;
import com.localtransfer.BuildConfig;
import com.localtransfer.Progress;
import com.localtransfer.R;
import com.localtransfer.Transfer;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class DownloadFragment extends Fragment {

    public DownloadFragment() {
        // Required empty public constructor
    }

    private View root;

    public static DownloadFragment newInstance() {
        DownloadFragment fragment = new DownloadFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        root = inflater.inflate(R.layout.fragment_download, container, false);

        new Timer().schedule(new TimerTask() {
            @Override
            public void run() {
                showFileList();
            }
        }, 100);

        SwipeRefreshLayout refresh = root.findViewById(R.id.refresh);

        refresh.setOnRefreshListener(() -> {
            new Thread(() -> showFileList()).start();
            refresh.setRefreshing(false);
        });

        return root;
    }

    @Override
    public void onPause(){
        super.onPause();

    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();

    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        menu.add ( 0 , v.getId (), 0 , "Delete" );
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        if (item.getTitle() == "Delete") {
            LinearLayout fileDesc = (LinearLayout) root.findViewById(item.getItemId());
            final String file = String.valueOf(fileDesc.getTag(R.id.ID_FILE_NAME));
            new Thread(() -> {
                try {
                    Transfer tr = new Transfer();
                    String message = tr.deleteFile(file);
                    System.out.println(message);
                    Transfer.activity.runOnUiThread(() ->
                            Snackbar.make(getActivity().findViewById(R.id.view_pager), message, Snackbar.LENGTH_LONG)
                                    .setAction("Action", null).show());
                    showFileList();
                } catch (IOException e) {
                    String ExceptionName = e.getClass().getSimpleName();
                    String ExceptionMess = e.getMessage();

                    if(ExceptionName != null && ExceptionMess != null) {
                        Transfer.error(ExceptionName + ": " + ExceptionMess, null, null);
                    }

                }
            }).start();
        }
        else {
            return  false;
        }
        return true;
    }

    public void showFileList() {

        final TextView title = new TextView(Transfer.activity);
        title.setTextSize(18);
        title.setGravity(Gravity.CENTER);
        final LinearLayout main_layout = root.findViewById(R.id.main_layout);
        final LayoutInflater inflater = (LayoutInflater) getActivity().getSystemService(getContext().LAYOUT_INFLATER_SERVICE);

        Transfer tr = new Transfer();

        String data = "";

        try {
            data = tr.getFileList();
        } catch (IOException e) {
            final String ExceptionName = e.getClass().getSimpleName();
            final String ExceptionMess = e.getMessage();

            if(ExceptionName != null && ExceptionMess != null) {
                Transfer.error(ExceptionName + ": " + ExceptionMess, title, main_layout);
            }

        }
        if (data.equals("[]")) {
            Transfer.activity.runOnUiThread(() -> {
                main_layout.removeAllViews();
                main_layout.addView(title);
                title.setHeight(1000);
                title.setText("No file to download");
            });
            return;
        }

        try {

            JSONArray array;
            array = new JSONArray(data);

            Transfer.activity.runOnUiThread(() -> {
                main_layout.removeAllViews();
                final View line = inflater.inflate(R.layout.horizontal_line, null);
                main_layout.addView(line);
            });
            for (int i = 0; i < array.length(); i++) {
                JSONObject row = array.getJSONObject(i);
                final String name = row.getString("name");
                final String href = row.getString("href");
                final long size = row.getLong("size");
                final String mime = row.getString("mime");
                final String type = row.getString("type");

                Transfer.activity.runOnUiThread(() -> {
                    title.setText("Choose file to download");
                    View file_info = inflater.inflate(R.layout.file_info, null);
                    main_layout.addView(file_info);
                    ImageView image = file_info.findViewById(R.id.file_image);
                    TextView viewName = file_info.findViewById(R.id.file_name);
                    TextView viewType = file_info.findViewById(R.id.file_type);
                    TextView viewSize = file_info.findViewById(R.id.file_size);
                    LinearLayout fileDesc = file_info.findViewById(R.id.file_desc);
                    LinearLayout file_buttons = file_info.findViewById(R.id.file_buttons);

                    fileDesc.setId(View.generateViewId());
                    file_buttons.setId(View.generateViewId());
                    file_buttons.setTag(R.id.ID_FILE_BUTTONS, "FOR VISIBILITY");

                    registerForContextMenu(fileDesc);
                    fileDesc.setTag(R.id.ID_FILE_NAME, name);

                    viewName.setText(name);
                    viewType.setText(mime);
                    viewSize.setText(Transfer.humanReadableByteCountBin(size));

                    switch (type) {
                        case "file-image":
                            image.setImageResource(R.drawable.ic_icon_image);
                            break;
                        case "file-audio":
                            image.setImageResource(R.drawable.ic_icon_music);
                            break;
                        case "file-video":
                            image.setImageResource(R.drawable.ic_icon_video);
                            break;
                        default:
                            image.setImageResource(R.drawable.ic_icon_file);
                    }

                    fileDesc.setOnClickListener(view -> {
                        String save_location;

                        if (mime.equals("text/plain"))
                            save_location = getContext().getCacheDir().toString();
                        else
                            save_location = Environment.getExternalStorageDirectory() + "/" + Transfer.local_storage;

                        File file = new File(save_location, name);

                        file_buttons.setId(View.generateViewId());

                        Button Bview = file_buttons.findViewById(R.id.file_view);
                        Button Bshare = file_buttons.findViewById(R.id.file_share);
                        Button dl = file_buttons.findViewById(R.id.file_download);

                        dl.setOnClickListener(ListenerDL);
                        Bview.setOnClickListener(ListenerView);
                        Bshare.setOnClickListener(ListenerShare);

                        if (file.exists() && file.length() == size) {
                            Bview.setVisibility(LinearLayout.VISIBLE);
                            Bshare.setVisibility(LinearLayout.VISIBLE);
                            dl.setVisibility(LinearLayout.GONE);
                        }
                        else {
                            dl.setVisibility(LinearLayout.VISIBLE);
                            Bview.setVisibility(LinearLayout.GONE);
                            Bshare.setVisibility(LinearLayout.GONE);
                        }

                        if (Progress.setButton(name, dl)) {
                            dl.setEnabled(false);
                            final Drawable spinner = Transfer.activity.getDrawable(R.drawable.ic_spinner_rotate);
                            int h = spinner.getIntrinsicHeight();
                            int w = spinner.getIntrinsicWidth();
                            spinner.setBounds(0, 0, w, h);
                            dl.setCompoundDrawables(dl.getCompoundDrawables()[0], null, spinner, null);
                            ObjectAnimator anim = ObjectAnimator.ofInt(spinner, "level", 0, 10000);
                            anim.setDuration(1000);
                            anim.setRepeatCount(Animation.INFINITE);
                            anim.setInterpolator(new LinearInterpolator());
                            anim.start();
                        }

                        file_buttons.setTag(R.id.ID_FILE_NAME, name);
                        file_buttons.setTag(R.id.ID_FILE_SIZE, size);
                        file_buttons.setTag(R.id.ID_FILE_HREF, href);
                        file_buttons.setTag(R.id.ID_FILE_MIME, mime);
                        file_buttons.setTag(R.id.ID_SAVE_LOCATION, save_location);
                        int state = file_buttons.getVisibility();
                        if(state == LinearLayout.GONE) {
                            for(View child : getAllChildren(main_layout)) {
                                if(child.getTag(R.id.ID_FILE_BUTTONS) != null)
                                    child.setVisibility(LinearLayout.GONE);
                            }
                            file_buttons.setVisibility(LinearLayout.VISIBLE);
                        }
                        else if(state == LinearLayout.VISIBLE)
                            file_buttons.setVisibility(LinearLayout.GONE);
                    });

                });
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    private View.OnClickListener ListenerDL = v -> {
        v.setEnabled(false);
        final LinearLayout layout = (LinearLayout) v.getParent();
        final Button button = layout.findViewById(R.id.file_download);

        final String name = (String) layout.getTag(R.id.ID_FILE_NAME);
        final String save_location = String.valueOf(layout.getTag(R.id.ID_SAVE_LOCATION));
        final long fileSize = (long) layout.getTag(R.id.ID_FILE_SIZE);
        final String href = (String) layout.getTag(R.id.ID_FILE_HREF);

        new File(save_location).mkdirs();
        Uri uri = Uri.fromFile(new File(save_location, name));

        Transfer tr = new Transfer();

        new Thread(() -> {
            try {
                tr.downloadFile(uri, name, fileSize, href, button);
            } catch (IOException e) {
                final String ExceptionName = e.getClass().getSimpleName();
                final String ExceptionMess = e.getMessage();

                if(ExceptionName != null && ExceptionMess != null) {
                    Transfer.error(ExceptionName + ": " + ExceptionMess, null, null);
                }

            }
        }).start();
    };

    private View.OnClickListener ListenerShare = v -> {
        LinearLayout layout = (LinearLayout) v.getParent();
        final String name = (String) layout.getTag(R.id.ID_FILE_NAME);
        final String type = (String) layout.getTag(R.id.ID_FILE_MIME);

        final String save_location = String.valueOf(layout.getTag(R.id.ID_SAVE_LOCATION));
        File file = new File(save_location, name);
        Uri uri = FileProvider.getUriForFile(getContext(), BuildConfig.APPLICATION_ID, file);

        if(type.equals("text/plain")) {
            String text = null;
            try {
                InputStream is = new FileInputStream(file);
                BufferedReader buf = new BufferedReader(new InputStreamReader(is));

                String line;
                StringBuilder sb = new StringBuilder();

                while((line = buf.readLine()) != null) {
                    sb.append(line).append("\n");
                }

                text = sb.toString();
            } catch (IOException e) {
                e.printStackTrace();
            }


            try {
                Intent intent = new Intent(Intent.ACTION_SEND);
                intent.setType(type);
                intent.putExtra(Intent.EXTRA_TEXT, text);
                startActivity(Intent.createChooser(intent, getString(R.string.share_title)));
            } catch (ActivityNotFoundException e) {
                e.printStackTrace();
            }
        }
        else {
            try {
                Intent intent = new Intent(Intent.ACTION_SEND);
                intent.putExtra(Intent.EXTRA_STREAM, uri);
                intent.setType(type);
                intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                startActivity(Intent.createChooser(intent, getString(R.string.share_title)));
            } catch (ActivityNotFoundException e) {
                e.printStackTrace();
            }
        }


    };

    private View.OnClickListener ListenerView = v -> {
        LinearLayout layout = (LinearLayout) v.getParent();
        final String name = (String) layout.getTag(R.id.ID_FILE_NAME);
        final String type = (String) layout.getTag(R.id.ID_FILE_MIME);

        final String save_location = String.valueOf(layout.getTag(R.id.ID_SAVE_LOCATION));
        File file = new File(save_location, name);
        Uri uri = FileProvider.getUriForFile(getContext(), BuildConfig.APPLICATION_ID, file);

        try {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setDataAndType(uri, type);
            intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            startActivity(intent);
        } catch (ActivityNotFoundException e) {
            e.printStackTrace();
        }


    };

    private List<View> getAllChildren(View v) {

        if (!(v instanceof ViewGroup)) {
            ArrayList<View> viewArrayList = new ArrayList<>();
            viewArrayList.add(v);
            return viewArrayList;
        }

        ArrayList<View> result = new ArrayList<>();

        ViewGroup viewGroup = (ViewGroup) v;
        for (int i = 0; i < viewGroup.getChildCount(); i++) {

            View child = viewGroup.getChildAt(i);

            //add child
            result.add(child);

            //Do the same with child of child
            result.addAll(getAllChildren(child));
        }
        return result;
    }

}